<?php

namespace QUI\RabbitMQServer\REST;

use PCSG\Serializer\EntitySerializer;
use PCSG\Serializer\EntitySerializerInterface;
use Psr\Http\Message\ServerRequestInterface as RequestInterface;
use QUI;
use QUI\RabbitMQServer\REST\Exception\InvalidRequestDataException;
use QUI\RabbitMQServer\REST\Exception\ServerErrorException;
use QUI\REST\Utils\RequestUtils;

use function class_exists;
use function json_validate;

class RequestFieldDeserializer
{
    public function __construct(
        private readonly RequestInterface $request,
        private ?EntitySerializerInterface $entitySerializer = null
    ) {
        if (is_null($this->entitySerializer)) {
            $this->entitySerializer = new EntitySerializer();
        }
    }

    /**
     * Get instance of a specific class from request data.
     *
     * @param string $className
     *
     * @return object
     * @throws InvalidRequestDataException
     * @throws ServerErrorException
     */
    public function getInstanceFromRequest(
        string $className
    ): object {
        if (!class_exists($className)) {
            throw new ServerErrorException(
                "Class '$className' does not exist."
            );
        }

        $this->request->getBody()->rewind();

        try {
            return $this->entitySerializer->deserialize($className, $this->request->getBody()->getContents());
        } catch (\Exception $exception) {
            QUI\System\Log::writeException($exception);
            throw new InvalidRequestDataException(
                "Request data invalid."
                . " Cannot deserialize to instance of '$className' :: " . $exception->getMessage(),
                0,
                [
                    'requestUri' => $this->request->getUri()
                ]
            );
        }
    }

    /**
     * Get instance of a specific class from request field data.
     *
     * @param string $fieldName
     * @param string $className
     *
     * @return object
     * @throws InvalidRequestDataException
     * @throws ServerErrorException
     */
    public function getInstanceFromRequestField(
        string $fieldName,
        string $className
    ): object {
        if (!class_exists($className)) {
            throw new ServerErrorException(
                "Class '$className' does not exist."
            );
        }

        $fieldData = RequestUtils::getFieldFromRequest($this->request, $fieldName);

        if ($fieldData === false) {
            throw new InvalidRequestDataException(
                "Request field '$fieldName' was not found in request data.",
                0,
                [
                    'requestUri' => $this->request->getUri()
                ]
            );
        }

        if (!json_validate($fieldData)) {
            throw new InvalidRequestDataException(
                "Request data in field '$fieldName' does not contain valid JSON."
                . " Cannot deserialize to instance of '$className'.",
                0,
                [
                    'requestUri' => $this->request->getUri()
                ]
            );
        }

        try {
            return $this->entitySerializer->deserialize($className, $fieldData);
        } catch (\Exception $exception) {
            QUI\System\Log::writeException($exception);
            throw new InvalidRequestDataException(
                "Request data in field '$fieldName' invalid."
                . " Cannot deserialize to instance of '$className' :: " . $exception->getMessage(),
                0,
                [
                    'requestUri' => $this->request->getUri()
                ]
            );
        }
    }
}
