<?php

namespace QUI\RabbitMQServer\REST\Route;

use Psr\Http\Message\ResponseInterface as ResponseInterface;
use Psr\Http\Message\ServerRequestInterface as RequestInterface;
use QUI\RabbitMQServer\REST\Exception\InvalidRequestDataException;
use QUI\RabbitMQServer\REST\Exception\ServerErrorException;
use QUI\RabbitMQServer\REST\RequestFieldDeserializer;
use QUI\RabbitMQServer\REST\Utils;
use Symfony\Component\HttpFoundation\Response as SymfonyResponse;

abstract class AbstractRoute
{
    protected readonly RequestInterface $request;
    protected readonly ResponseInterface $response;
    protected RequestFieldDeserializer $requestFieldDeserializer;

    final public function __invoke(RequestInterface $request, ResponseInterface $response): ResponseInterface
    {
        $this->request = $request;
        $this->response = $response;
        $this->requestFieldDeserializer = new RequestFieldDeserializer($request);
        return $this->handleRequest();
    }

    abstract protected function handleRequest(): ResponseInterface;

    /**
     * Get instance of a specific class from request data.
     *
     * @param string $className
     *
     * @return object
     * @throws InvalidRequestDataException
     * @throws ServerErrorException
     */
    final protected function getInstanceFromRequest(
        string $className
    ): object {
        return $this->requestFieldDeserializer->getInstanceFromRequest($className);
    }

    /**
     * Get instance of a specific class from request field data.
     *
     * @param string $fieldName
     * @param string $className
     *
     * @return object
     * @throws InvalidRequestDataException
     * @throws ServerErrorException
     */
    final protected function getInstanceFromRequestField(
        string $fieldName,
        string $className
    ): object {
        return $this->requestFieldDeserializer->getInstanceFromRequestField($fieldName, $className);
    }

    /**
     * Set data to successful response and return it.
     *
     * @param string|array $msg
     * @param int $statusCode (optional) - [default: 200]
     * @return ResponseInterface
     */
    final protected function buildSuccessResponse(
        string|array $msg,
        int $statusCode = SymfonyResponse::HTTP_OK
    ): ResponseInterface {
        return Utils::buildSuccessResponse($this->response, $msg, $statusCode);
    }

    /**
     * Set data to error response and return it.
     *
     * @param string|array $msg
     * @param int $errorCode
     * @param int $statusCode (optional) - [default: 200]
     * @return ResponseInterface
     */
    final protected function buildErrorResponse(
        string|array $msg,
        int $errorCode,
        int $statusCode = SymfonyResponse::HTTP_INTERNAL_SERVER_ERROR,
    ): ResponseInterface {
        return Utils::buildErrorResponse($this->response, $msg, $errorCode, $statusCode);
    }

    /**
     * Set data to error response and return it.
     *
     * @param \Exception $exception
     * @return ResponseInterface
     */
    final protected function buildExceptionErrorResponse(
        \Exception $exception
    ): ResponseInterface {
        return Utils::buildErrorResponse($this->response, $exception->getMessage(), $exception->getCode());
    }
}
