<?php

namespace QUI\RabbitMQServer;

use DateTimeImmutable;
use PCSG\Tasks\TaskSerializerInterface;
use PCSG\Tasks\WorkerTask;
use PCSG\Tasks\WorkerTaskDataInterface;
use QUI;
use QUI\QueueManager\QueueJob;
use QUI\QueueManager\QueueJobUUID;
use QUI\Utils\Uuid;

class TaskScheduler
{
    /**
     * Special data keys for worker task queue jobs.
     */
    const string JOB_DATA_KEY_WORKER_TASK_SERIALIZED = 'workerTask';
    const string JOB_DATA_KEY_WORKER_TASK_DATA_CLASS = 'workerTaskDataClass';
    const string JOB_DATA_KEY_CLONED_JOB_UUIDS = 'clonedJobUuids';

    /**
     * @param Server $server
     * @param TaskSerializerInterface $taskSerializer
     */
    public function __construct(
        private readonly Server $server,
        private readonly TaskSerializerInterface $taskSerializer
    ) {
    }

    /**
     * Create a queue job for a (QUIQQER-independent) task worker and queue it.
     *
     * @param WorkerTaskDataInterface $taskData
     * @param string $logLevel - PSR-3 log level
     * @param DateTimeImmutable|null $earliestQueueDate (optional) - If omitted, the job will be queued at the earliest
     * @param int|null $priority (optional) - default: 1 (standard priority)
     * @return QueueJobUUID - Worker task / queue job UUID
     *
     * @throws QUI\Exception
     */
    public function queueWorkerTask(
        WorkerTaskDataInterface $taskData,
        string $logLevel,
        ?DateTimeImmutable $earliestQueueDate = null,
        ?int $priority = 1
    ): string {
        $uuid = Uuid::get();

        $workerTask = new WorkerTask(
            $uuid,
            $taskData,
            $logLevel,
            $earliestQueueDate
        );

        $queueJob = $this->createQueueJobFromWorkerTask($workerTask);

        $queueJob->jobAttributes->earliestQueueDate = $earliestQueueDate;
        $queueJob->jobAttributes->priority = $priority;

        return $this->server->queueJob($queueJob);
    }

    /**
     * @param WorkerTask $workerTask
     * @return QueueJob
     */
    public function createQueueJobFromWorkerTask(WorkerTask $workerTask): QueueJob
    {
        $queueJob = new QueueJob(
            new QueueJobUUID($workerTask->uuid)
        );

        $queueJob->data->set(
            self::JOB_DATA_KEY_WORKER_TASK_SERIALIZED,
            $this->taskSerializer->serializeWorkerTaskForQueue($workerTask)
        );

        $queueJob->data->set(
            self::JOB_DATA_KEY_WORKER_TASK_DATA_CLASS,
            $workerTask->taskData::class
        );

        return $queueJob;
    }
}
